package com.portlandwebworks.commons.template;

import com.portlandwebworks.commons.domain.User;
import org.apache.commons.lang.LocaleUtils;
import org.apache.commons.lang.StringUtils;

import java.io.InputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;


/**
 * This software is the property of Portland Webworks.
 *
 * This provider
 *
 * @author Ronnie Northup
 */
public class LocaleBasedVelocityTemplateProvider extends VelocityTemplateProvider
{
	private HashMap<String, Boolean> resourceLookups = new HashMap<String, Boolean>();

	@Override
	protected String resolveTemplatePath(String templatePath, User user)
	{
		Locale locale = user == null || StringUtils.isBlank(user.getLocale())? Locale.getDefault() :  LocaleUtils.toLocale(user.getLocale());

		// Attempt to resolve the template by first trying the most locale-specific resource name possible, and if the
		// resource is not found keep reducing the specificity of the template until it is found.
		String resourcePath = templateDirectory + "/" + templatePath + "_" + locale.getLanguage() + "_" + locale.getCountry() + ".vm";

		if (! findResourceUsingClassLoader(resourcePath)) {

			resourcePath = templateDirectory + "/" + templatePath + "_" + locale.getLanguage() + ".vm";
			if (! findResourceUsingClassLoader(resourcePath)) {

				resourcePath = templateDirectory + "/" + templatePath + ".vm";
				if (! findResourceUsingClassLoader(resourcePath)) {
					throw new RuntimeException("Could not find template " + templatePath + " for any locale");
				}
			}
		}

		return resourcePath;
	}

	protected boolean findResourceUsingClassLoader(String resourcePath)
	{
		Boolean found = resourceLookups.get(resourcePath);
		if (found == null) {
			found = (null != this.getClass().getResource(resourcePath));
			resourceLookups.put(resourcePath, found);
		}
		return found;
	}
}
